;;;; web-unity-fcgi.scm
;
; Copyright (c) 2007, Peter Bex
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.

(use fastcgi web-unity)

(define wu:fcgi-environment (make-parameter #f))
(define wu:user-dispatcher (make-parameter #f))

(define (wu:run-handler d)
  (parameterize ((wu:user-dispatcher d))
   (fcgi-dynamic-server-accept-loop wu:fcgi-handler)))

;; XXX make some kind of parameter or something so people can specify how
;; to invoke their FCGI handler.
#;(define (wu:run-handler d)
  (parameterize ((wu:user-dispatcher d))
   (fcgi-accept-loop "/tmp/fastcgi-socket-0" 0 wu:fcgi-handler)))

(define (wu:fcgi-get-method-params method in out err env)
  (case method
    ((get)  (parse-encoded-arguments (alist-ref "QUERY_STRING" (env) equal? "")))
    ((post) (parse-encoded-arguments (fcgi-get-post-data in env)))
    (else   '())))

(define (wu:get-header header #!optional default)
  (alist-ref (string-append "HTTP_" (string-translate header #\- #\_)) (wu:fcgi-environment) string-ci=? default))

(define (wu:write-headers)
  (printf "Status: ~A ~A\r\n" (car (wu:response-code)) (cdr (wu:response-code)))
  (for-each (lambda (pair)
	    (printf "~A: ~A\r\n" (car pair) (cdr pair)))
	  (wu:response-headers))
  (printf "\r\n")
  (wu:headers-sent #t))

(define (wu:fcgi-handler in out err env)
  (let ((m (string->symbol (string-downcase (alist-ref "REQUEST_METHOD" (env) equal? "")))))
    (parameterize ((current-output-port (make-output-port out void))
		   (current-error-port (make-output-port err void))
		   (wu:params (wu:fcgi-get-method-params m in out err env))
		   (wu:request-method m)
		   (wu:query-params (parse-encoded-arguments (alist-ref "QUERY_STRING" (env) equal? "")))
		   (wu:path-info (alist-ref "PATH_INFO" (env) equal? ""))
		   (wu:script-name (alist-ref "SCRIPT_NAME" (env) equal? ""))
		   (wu:script-filename (alist-ref "SCRIPT_FILENAME" (env) equal? ""))
		   (wu:request-method m)
		   (wu:fcgi-environment (env))
		   (wu:response-headers (list (cons "Content-Type" "text/html")))
		   (wu:response-code (cons 200 "OK")))
      (wu:invoke-dispatcher (wu:user-dispatcher)))))
