;;;; web-unity-spiffy.scm
;
; Copyright (c) 2007, Peter Bex
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.

(use web-unity)

(define (wu:make-params)
  (case (string->symbol (string-downcase (symbol->string (http:request-method (current-request)))))
    ((get)  (current-urlencoded-arguments))
    ((post) (http:request-body (current-request)))
    (else   '())))

(define (wu:get-header header #!optional default)
  (alist-ref header (http:request-attributes (current-request)) string-ci=? default))

(define wu:original-output-port (make-parameter #f))
(define wu:original-error-port  (make-parameter #f))

(define (wu:write-headers)
  (parameterize ((current-response-headers (wu:response-headers))
		 (current-response-code (wu:response-code))
		 (current-output-port (wu:original-output-port))
		 (current-error-port  (wu:original-error-port)))
    (write-response-header)
    (wu:headers-sent #t)))

;; Try to find out the most efficient way of outputting stuff
(define (wu:possibly-chunked dispatcher)
  (if (eq? (http:request-protocol (current-request)) 'HTTP/1.1)
      (parameterize ((current-output-port (wu:chunked-output-port (current-output-port)))
		     (current-error-port (wu:chunked-output-port (current-error-port))))
	(wu:set-header! "Transfer-Encoding" "chunked")
	(dispatcher)
	(fprintf (wu:original-output-port) "0\r\n\r\n"))
      (begin
	(http:force-close #t)
	(dispatcher))))

(define (wu:run-handler dispatcher)
  ((web-unity-return-dispatcher)
   (lambda (filename)
     (parameterize ((wu:params (wu:make-params))
		    (wu:request-method (string->symbol (string-downcase (symbol->string (http:request-method (current-request))))))
		    (wu:query-params (current-urlencoded-arguments))
		    (wu:path-info (current-pathinfo))
		    (wu:script-name (string-trim-right (string-chomp (current-path) (current-pathinfo)) #\/))
		    (wu:script-filename filename)
		    (wu:response-headers (current-response-headers))
		    (wu:response-code (current-response-code))
		    (wu:original-output-port (current-output-port))
		    (wu:original-error-port (current-error-port)))
      (wu:possibly-chunked (lambda () (wu:invoke-dispatcher dispatcher)))))))
